import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.actionSystem.DataConstants;
import com.intellij.openapi.actionSystem.DataContext;
import com.intellij.openapi.diagnostic.Logger;
import com.intellij.openapi.diff.DiffManager;
import com.intellij.openapi.diff.DiffTool;
import com.intellij.openapi.diff.SimpleContent;
import com.intellij.openapi.diff.SimpleDiffRequest;
import com.intellij.openapi.project.Project;
import com.intellij.openapi.ui.Messages;
import intellijmaven.core.Dependency;
import intellijmaven.core.IntelliJModule;
import intellijmaven.core.MavenProject;

import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;

/**
 * Lauches the comparison tool to display differences between the IDEA and the Maven dependencies.
 * Creates two temporary files to do the comparsion. This is needed if a 3rd party comparison
 * tool is configured.
 *
 * @author Jeff Sabin
 */
public class CompareDependencies extends AnAction
{
    private static final Logger logger = Logger.getInstance(CompareDependencies.class.getName());
    private static final String DIFF_REQUEST_TITLE = "Dependency Comparison";
    private static final String IDEA_COMPARE_TITLE = "IDEA Dependencies";
    private static final String MAVEN_COMPARE_TITLE = "Maven Dependencies";

    public void actionPerformed(final AnActionEvent e)
    {
        DataContext dataContext = e.getDataContext();
        Project project = (Project)dataContext.getData(DataConstants.PROJECT);

        MavenUpdatorComponent component = project.getComponent(MavenUpdatorComponent.class);

        try
        {
            MavenProject mavenProject;
            IntelliJModule module;

            mavenProject = component.getMavenProject();
            module = component.getModule();

//            SimpleContent ideaContent = new SimpleContent(createContent(module.getDependencies()),
//                    StdFileTypes.PLAIN_TEXT);
//            SimpleContent mavenContent = new SimpleContent(createContent(mavenProject.getDependencies()),
//                    StdFileTypes.PLAIN_TEXT);
            SimpleContent ideaContent = new SimpleContent(createContent(module.getDependencies()));
            SimpleContent mavenContent = new SimpleContent(createContent(mavenProject.getDependencies()));
            ideaContent.setReadOnly(false);
            mavenContent.setReadOnly(false);

            SimpleDiffRequest request = new SimpleDiffRequest(project, DIFF_REQUEST_TITLE);
            request.setContentTitles(MAVEN_COMPARE_TITLE, IDEA_COMPARE_TITLE);
            request.setContents(mavenContent, ideaContent);

            request.addHint(DiffTool.HINT_SHOW_MODAL_DIALOG);
            request.addHint(DiffTool.HINT_SHOW_FRAME);

            DiffManager diffManager = DiffManager.getInstance();
            DiffTool diffTool = diffManager.getIdeaDiffTool();
            diffTool.show(request);

//            Messages.showYesNoDialog(project, "Do you wish to save changes?", "Save changes", Messages.getInformationIcon());
//            System.out.println("differneces = " + request.getContents()[0].getDocument().getText());
//            System.out.println("differneces = " + request.getContents()[1].getDocument().getText());
        }
        catch (Exception ex)
        {
            logger.error("", ex);
            Messages.showErrorDialog(project, "Error retrieving project and module. " + ex, "Error");
        }
    }

    private String createContent(List dependencies)
    {
        Collections.sort(dependencies, new DependencyComparator());
        StringBuffer buffer = new StringBuffer();

        for(Iterator iterator = dependencies.iterator(); iterator.hasNext();)
        {
            Dependency dependency = (Dependency)iterator.next();
            buffer.append("Group Id:\t\t" + dependency.getGroupId());
            buffer.append("Artifact Id:\t" + dependency.getArtifactId());
            buffer.append("Version:\t\t" + dependency.getVersion());
            buffer.append("\n\n");
        }

        return buffer.toString();
    }

//    private File createDependencyFiles(List dependencies) throws IOException
//    {
//        File file = File.createTempFile(TMP_FILE_NAME, TMP_FILE_EXTENSION);
//        PrintWriter writer = new PrintWriter(new FileWriter(file));
//
//        try
//        {
//            Collections.sort(dependencies, new DependencyComparator());
//            for(Iterator iterator = dependencies.iterator(); iterator.hasNext();)
//            {
//                Dependency dependency = (Dependency)iterator.next();
//                writer.println("Group Id:\t\t" + dependency.getGroupId());
//                writer.println("Artifact Id:\t" + dependency.getArtifactId());
//                writer.println("Version:\t\t" + dependency.getVersion());
//                writer.println();
//                writer.println();
//            }
//        }
//        finally
//        {
//            writer.close();
//        }
//        return file;
//    }

    private class DependencyComparator implements Comparator
    {
        public int compare(Object o, Object o1)
        {
            Dependency d1 = (Dependency)o;
            Dependency d2 = (Dependency)o;

            return d1.getArtifactId().compareToIgnoreCase(d2.getArtifactId());
        }
    }
}
