package intellijmaven.core;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import java.io.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * @author Jeff Sabin
 */
public class IntelliJModule
{
    private File moduleFile;
    private List dependencies;
    private boolean isWebModule;
    private Document document;

    public IntelliJModule(File moduleFile) throws IOException, JDOMException
    {
        this.moduleFile = moduleFile;

        dependencies = readDependencies(new FileInputStream(moduleFile));
    }

    public boolean isWebModule()
    {
        return isWebModule;
    }

    public File getModuleFile()
    {
        return moduleFile;
    }

    /**
     * Read all dependencies from an IntelliJ module.
     */
    private List readDependencies(InputStream stream) throws IOException, JDOMException
    {
        List dependencies = new ArrayList();
        SAXBuilder builder = new SAXBuilder();
        document = builder.build(stream);
        Element rootElement = document.getRootElement();

        Element moduleRootManager = getNamedChild(rootElement, Constants.TAG_MODULE_ROOT_MANAGER);
        assert moduleRootManager != null;

        Element webProperties = getNamedChild(rootElement, "WebModuleProperties");
        isWebModule = webProperties != null;


        List children = moduleRootManager.getChildren();
        for(Iterator iterator = children.iterator(); iterator.hasNext();)
        {
            Element child = (Element)iterator.next();

            if(Constants.TAG_ORDERENTRY.equals(child.getName()) && Constants.ATTRIBUTE_MODULE_LIBRARY.equals(child.getAttributeValue(Constants.ATTRIBUTE_TYPE)))
            {
                // get library
                Element library = child.getChild(Constants.TAG_LIBRARY);
                assert library != null;

                Element classes = library.getChild(Constants.TAG_CLASSES);
                assert classes != null;

                Element root = classes.getChild(Constants.TAG_ROOT);
                assert root != null;

                IIntelliJUrl url = new IntelliJUrl(root.getAttributeValue(Constants.ATTRIBUTE_URL));

                if(url.getGroupId() == null)
                {
                    System.err.println("WARNING: intellijmaven.core.Dependency is not in the Maven repository: " + url.toString());
                }
                else
                {
                    Dependency dependency = new Dependency(url.getGroupId(), url.getArtifactId(), url.getVersion());
                    dependencies.add(dependency);

                    // Remove dependency
                    iterator.remove();
                }
            }
        }
        return dependencies;
    }

    private Element getNamedChild(Element parent, String name)
    {
        java.util.List children = parent.getChildren(Constants.TAG_COMPONENT);
        for(Iterator iterator = children.iterator(); iterator.hasNext();)
        {
            Element child = (Element)iterator.next();
            if(child.getAttributeValue(Constants.ATTRIBUTE_NAME).equals(name))
            {
                return child;
            }
        }

        return null;
    }


    /**
     * Returns a list of all dependencies.
     *
     * @return list of {@link Dependency}
     */
    public List getDependencies()
    {
        return dependencies;
    }

    /**
     * Adds the dependency to the list of all dependencies. If it already exist it is not added
     * again.
     */
    public void addDependency(Dependency dependency)
    {
        if(!dependencies.contains(dependency))
        {
            dependencies.add(dependency);
        }
    }

    /**
     * Removes all dependencies.
     */
    public void removeAllDependencies()
    {
        dependencies = new ArrayList();
    }

    /**
     * Saves changes to disk.
     */
    public void save() throws IOException, JDOMException
    {
        writeDependencies();

        XMLOutputter outputter = new XMLOutputter();
        outputter.setFormat(Format.getPrettyFormat());
        OutputStream stream = new FileOutputStream(moduleFile);
        outputter.output(document, stream);
        stream.close();

    }

    private void writeDependencies()
    {
        // IML syntax is the following
        //  <component name="NewModuleRootManager">
        //    <orderEntry type="module-library">
        //      <library name="commons-beanutils">
        //        <CLASSES>
        //          <root url="jar://C:/Documents and Settings/jsabin/.maven/repository/commons-beanutils/jars/commons-beanutils-1.7.0.jar!/" />
        //        </CLASSES>
        //        <JAVADOC />
        //        <SOURCES />
        //      </library>
        //    </orderEntry>
        //  </component>

        Element moduleRootManager = getNamedChild(document.getRootElement(), Constants.TAG_MODULE_ROOT_MANAGER);
        assert moduleRootManager != null;

        Element moduleProperties = getNamedChild(document.getRootElement(), Constants.TAG_MODULE_PROPERTIES);

        for(Iterator iterator = dependencies.iterator(); iterator.hasNext();)
        {
            IDependency dependency = (IDependency)iterator.next();


            Element entry = new Element(Constants.TAG_ORDERENTRY);
            entry.setAttribute(Constants.ATTRIBUTE_TYPE, Constants.ATTRIBUTE_MODULE_LIBRARY);
            moduleRootManager.addContent(entry);

            Element library = new Element(Constants.TAG_LIBRARY);
            entry.addContent(library);

            Element classes = new Element(Constants.TAG_CLASSES);
            library.addContent(classes);

            Element root = new Element(Constants.TAG_ROOT);
            root.setAttribute(Constants.ATTRIBUTE_URL, dependency.getUrl().toString());
            classes.addContent(root);

            // todo with Maven 2 we can add sources and javadoc
            Element javadoc = new Element(Constants.TAG_JAVADOC);
            library.addContent(javadoc);

            Element sources = new Element(Constants.TAG_SOURCES);
            library.addContent(sources);

            // Add module property
            if(moduleProperties != null)
            {
                addModuleProperty(moduleProperties, dependency);
            }
        }
    }

    /**
     * Adds web based module properties.
     */
    private void addModuleProperty(Element moduleProperties, IDependency dependency)
    {
        // XML looks like this
        //  <component name="WebModuleProperties">
        //    <containerElement type="library" level="module">
        //      <attribute name="method" value="1" />
        //      <attribute name="URI" value="/WEB-INF/lib/dbtables-1.0-SNAPSHOT.jar" />
        //      <url>jar://C:/Documents and Settings/jsabin/.maven/repository/no.fast.marketrac/jars/dbtables-1.0-SNAPSHOT.jar!/</url>
        //    </containerElement>
        //  </component>

        Element container = new Element(Constants.TAG_CONTAINER_ELEMENT);
        container.setAttribute(Constants.ATTRIBUTE_TYPE, Constants.ATTRIBUTE_LIBRARY);
        container.setAttribute(Constants.ATTRIBUTE_LEVEL, Constants.ATTRIBUTE_MODULE);
        moduleProperties.addContent(container);

        Element method = new Element(Constants.TAG_ATTRIBUTE);
        method.setAttribute(Constants.ATTRIBUTE_NAME, Constants.ATTRIBUTE_METHOD);
        method.setAttribute(Constants.ATTRIBUTE_VALUE, "1"); // Copy to target
        container.addContent(method);

        Element uri = new Element(Constants.TAG_ATTRIBUTE);
        uri.setAttribute(Constants.ATTRIBUTE_NAME, Constants.ATTRIBUTE_URI);
        uri.setAttribute(Constants.ATTRIBUTE_VALUE, "/WEB-INF/lib/" + dependency.getArtifactId() + "-" +
                dependency.getVersion() + ".jar");
        container.addContent(uri);

        Element url = new Element(Constants.TAG_URL);
        url.addContent(dependency.getUrl().toString());
        container.addContent(url);
    }
}
