package intellijmaven.core;

import java.util.StringTokenizer;

/**
 * Used to parse IntelliJ urls and convert them to Maven pieces. The syntax of an IntelliJ url is
 * <p> jar://C:/Documents and Settings/jsabin/.maven/repository/no.fast.foundation/jars/toc-1.3-SNAPSHOT.jar!/
 *
 *  @author Jeff Sabin
 */
class IntelliJUrl implements IIntelliJUrl
{
    private String url;
    private static final String SNAPSHOT = "SNAPSHOT";
    private static final String REPOSITORY = "repository";

    private String version;
    private String groupId;
    private String artifact;

    public IntelliJUrl(String url)
    {
        this.url = url;
    }

    public String getArtifactId()
    {
        if (artifact == null)
        {

            // Get file name
            String fileName = null;
            StringTokenizer tokenizer = new StringTokenizer(url, "/");
            while(tokenizer.hasMoreTokens())
            {
                fileName = tokenizer.nextToken();
            }
            assert fileName != null;

            // Remove any extension
            int index = fileName.lastIndexOf(".");
            if(index > -1)
            {
                fileName = fileName.substring(0, index);
            }

            String version = getVersion();
            if (version != null)
            {
                int versionIndex = fileName.indexOf("-" + version);
                artifact = fileName.substring(0, versionIndex);
            }
            else
            {
                // No version
                artifact = fileName;
            }
        }

        return artifact;
    }

    public String getVersion()
    {
        if (version == null)
        {
            // Get file name
            String fileName = null;
            StringTokenizer tokenizer = new StringTokenizer(url, "/");
            while(tokenizer.hasMoreTokens())
            {
                fileName = tokenizer.nextToken();
            }
            assert fileName != null;

            // Remove any extension
            int index = fileName.lastIndexOf(".");
            if(index > -1)
            {
                fileName = fileName.substring(0, index);
            }

            String previousToken = null;
            tokenizer = new StringTokenizer(fileName, "-");
            while(tokenizer.hasMoreTokens())
            {
                previousToken = version;
                version = tokenizer.nextToken();
            }

            if(SNAPSHOT.equalsIgnoreCase(version))
            {
                version = previousToken + "-" + version;
            }

            // Warn the user if the version may not be a valid version number
            isVersionValid();
//            if (!isVersionValid())
//            {
//                version = null;
//            }

        }
        return version;
    }

    private boolean isVersionValid()
    {
        String verNum = version;

        // Strip off "-SNAPSHOT" if it exists. This is valid
        int index = verNum.indexOf("-" + SNAPSHOT);
        if (index > -1)
        {
            verNum = verNum.substring(0, index);
        }

        StringTokenizer tokenizer;
        // Verify that the version is a valid version number
        tokenizer = new StringTokenizer(verNum, ".");
        while(tokenizer.hasMoreElements())
        {
            String number = tokenizer.nextToken();
            try
            {
                if (SNAPSHOT.equals(number))
                {
                    // Snapshot is valid
                    continue;
                }
                Integer.parseInt(number);
            }
            catch (NumberFormatException e)
            {
                System.err.println("WARNING: version number may not be valid: " + version);
                return false;
            }
        }
        return true;
    }

    public String getGroupId()
    {
        if (groupId == null)
        {

            StringTokenizer tokenizer = new StringTokenizer(url, "/");
            while(tokenizer.hasMoreTokens())
            {
                String token = tokenizer.nextToken();
                if(token.equalsIgnoreCase(REPOSITORY))
                {
                    groupId = tokenizer.nextToken();
                    break;
                }
            }
        }

        return groupId;
    }


    public String toString()
    {
        return url;
    }
}