package intellijmaven.core;

import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import java.io.*;
import java.util.*;

/** @author Jeff Sabin */
public class MavenProject implements IMavenProject
{
	private static final String PROPERTIES_FILE_NAME = "mavenProject.properties";

	private Document document;
	private File projectFile;
	private List dependencies;
	private boolean isWebProject;
	private Properties properties;
	private static final String TEST_DEPEDENDENCIES = "test.depedendencies";


	public MavenProject(File projectFile, boolean webProject) throws IOException, JDOMException
	{
		this.projectFile = projectFile;
		isWebProject = webProject;

		properties = new Properties();
		File propertiesFile = new File(PROPERTIES_FILE_NAME);
		if (propertiesFile.exists())
		{
			properties.load(new FileInputStream(propertiesFile));
		}

		SAXBuilder builder = new SAXBuilder();
		document = builder.build(projectFile);
		dependencies = readDependencies();
	}

	public MavenProject(File projectFile) throws IOException, JDOMException
	{
		this(projectFile, false);
	}

	public File getProjectFile()
	{
		return projectFile;
	}

	/**
	 * Returns a list of all dependencies used only for testing.
	 *
	 * @return test dependencies
	 */
	private List getTestDependencyNames()
	{
		String testDependenciesProperty = properties.getProperty(TEST_DEPEDENDENCIES);
		List testDependencies = new ArrayList();

		if (testDependenciesProperty != null)
		{
			StringTokenizer tokenizer = new StringTokenizer(testDependenciesProperty, ", ");
			while (tokenizer.hasMoreTokens())
			{
				testDependencies.add(tokenizer.nextToken());
			}
		}

		return testDependencies;
	}

	private List readDependencies()
	{
		List dependencies = new ArrayList();
		Element dependenciesElement = document.getRootElement().getChild(Constants.TAG_DEPENDENCIES);

		if (dependenciesElement != null)
		{
			List children = dependenciesElement.getChildren();

			for (Iterator iterator = children.iterator(); iterator.hasNext();)
			{
				Element element = (Element)iterator.next();
				String artifact = element.getChildText(Constants.MAVEN_ARTIFACTID);
				String groupid = element.getChildText(Constants.MAVEN_GROUPID);
				String version = element.getChildText(Constants.MAVEN_VERSION);
				dependencies.add(new Dependency(groupid, artifact, version));

				// remove dependency
				iterator.remove();
			}
		}
		return dependencies;
	}

	public List getDependencies()
	{
		return dependencies;
	}


	public void addDependency(Dependency dependency)
	{
		if (!dependencies.contains(dependency))
		{
			dependencies.add(dependency);
		}
	}

	public void removeAllDependencies()
	{
		dependencies = new ArrayList();
	}

	public void save() throws IOException, JDOMException
	{
		writeDependencies();

		XMLOutputter outputter = new XMLOutputter();
		outputter.setFormat(Format.getPrettyFormat());
		OutputStream stream = new FileOutputStream(projectFile);
		outputter.output(document, stream);
		stream.close();
	}

	private void writeDependencies()
	{
		Element dependenciesElement = document.getRootElement().getChild(Constants.TAG_DEPENDENCIES);
		if (dependenciesElement == null)
		{
			// Create it
			dependenciesElement = new Element(Constants.TAG_DEPENDENCIES);
			document.getRootElement().addContent(dependenciesElement);
		}

		List testDependencies = getTestDependencyNames();

		// Sort dependencies by artifact id
		Collections.sort(dependencies, new DependencyComparator());

		// write dependencies
		for (Iterator iterator = dependencies.iterator(); iterator.hasNext();)
		{
			IDependency dependency = (IDependency)iterator.next();
			Element dependencyElement = new Element(Constants.TAG_DEPENDENCY);

			Element groupIdElement = new Element(Constants.MAVEN_GROUPID);
			groupIdElement.addContent(dependency.getGroupId());

			Element artifactIdElement = new Element(Constants.MAVEN_ARTIFACTID);
			artifactIdElement.addContent(dependency.getArtifactId());

			Element versionElement = new Element(Constants.MAVEN_VERSION);
			versionElement.addContent(dependency.getVersion());


			dependencyElement.addContent(groupIdElement);
			dependencyElement.addContent(artifactIdElement);
			dependencyElement.addContent(versionElement);

			if (isWebProject && !testDependencies.contains(dependency.getGroupId()))
			{
				Element propertiesElement = new Element(Constants.TAG_PROPERTIES);
				Element warBundleElement = new Element(Constants.TAG_WAR_BUNDLE);
				warBundleElement.addContent("true");

				propertiesElement.addContent(warBundleElement);
				dependencyElement.addContent(propertiesElement);
			}

			dependenciesElement.addContent(dependencyElement);
		}
	}

	public static void main(String[] args) throws IOException, JDOMException
	{
		IMavenProject project = new MavenProject(new File("C:/development/IntellijMaven/project2.xml"));

		project.addDependency(new Dependency("jar1", "Jeff", "1.0"));
		project.addDependency(new Dependency("jar2", "Jeff", "2.0"));
		project.addDependency(new Dependency("jar3", "Jeff", "3.0"));
		project.addDependency(new Dependency("jar4", "Jeff", "4.0"));

		project.save();
	}

	private class DependencyComparator implements Comparator
	{
		public int compare(Object o1, Object o2)
		{
			IDependency d1 = (IDependency)o1;
			IDependency d2 = (IDependency)o2;

			return d1.getArtifactId().compareToIgnoreCase(d2.getArtifactId());
		}
	}
}
