package intellijmaven.tools;

import intellijmaven.core.IDependency;
import intellijmaven.core.IntelliJModule;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.output.Format;
import org.jdom.output.XMLOutputter;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Creates a Maven project from an IntelliJ project.
 *
 * @author Jeff Sabin
 */
public class CreateProjectXML
{
    private File moduleFile;
    private List dependencies;
    private String extendingProject;
    private String name;
    private String version;
    private String shortDescription;
    private String longDescription;
    private String artifactId;
    private String repository;
    private boolean isWebModule;

    public CreateProjectXML(File moduleFile)
    {
        assert moduleFile != null;
        this.moduleFile = moduleFile;

        dependencies = new ArrayList();

        createAndShowGUI();
    }

    private void writeDependencies()
    {

        File projectFile = new File(moduleFile.getParent(), "project.xml");

        Document projectDoc = new Document();

        // Project tag
        Element projectElement = new Element("project");
        projectDoc.setRootElement(projectElement);

        // Extend tag
        Element extendElement = new Element("extend");
        projectElement.addContent(extendElement);
        extendElement.addContent(extendingProject);

        // ArtifactId tag
        Element artifactElement = new Element("artifactId");
        projectElement.addContent(artifactElement);
        artifactElement.addContent(artifactId);

        // Current version tag
        Element projectVersionElement = new Element("currentVersion");
        projectElement.addContent(projectVersionElement);
        projectVersionElement.addContent(version);

        // Name tag
        Element nameElement = new Element("name");
        projectElement.addContent(nameElement);
        nameElement.addContent(name);

        // Short Description tag
        Element shortDescriptionElement = new Element("shortDescription");
        projectElement.addContent(shortDescriptionElement);
        shortDescriptionElement.addContent(shortDescription);

        // Description tag
        Element longDescriptionElement = new Element("description");
        projectElement.addContent(longDescriptionElement);
        longDescriptionElement.addContent(longDescription);

        // Repository
        Element repositoryElement = new Element("repository");
        projectElement.addContent(repositoryElement);

        Element developerElement = new Element("developerConnection");
        repositoryElement.addContent(developerElement);
        developerElement.addContent(repository);

        Element dependenciesElement = new Element("dependencies");
        projectElement.addContent(dependenciesElement);

        for(Iterator iterator = dependencies.iterator(); iterator.hasNext();)
        {
            IDependency dependency = (IDependency) iterator.next();

            Element dependencyElement = new Element("dependency");
            dependenciesElement.addContent(dependencyElement);

            Element groupIdElement = new Element("groupId");
            groupIdElement.addContent(dependency.getGroupId());

            Element artifactIdElement = new Element("artifactId");
            artifactIdElement.addContent(dependency.getArtifactId());

            Element versionElement = new Element("version");
            versionElement.addContent(dependency.getVersion());

            dependencyElement.addContent(groupIdElement);
            dependencyElement.addContent(artifactIdElement);
            dependencyElement.addContent(versionElement);

            // Add to war if web project
            if(isWebModule)
            {
                Element properties = new Element("properties");
                dependencyElement.addContent(properties);

                Element war = new Element("war.bundle");
                war.addContent("true");
                properties.addContent(war);
            }
        }

        // Write it out
        try
        {
            FileOutputStream stream = new FileOutputStream(projectFile);
            XMLOutputter outputter = new XMLOutputter();
            outputter.setFormat(Format.getPrettyFormat());
            outputter.output(projectDoc, stream);
            stream.close();
        }
        catch (IOException e)
        {
            System.out.println("Error creating project file: " + projectFile);
            e.printStackTrace();
        }
    }

    private void createAndShowGUI()
    {
        //Create and set up the window.
        final JFrame frame = new JFrame("Create Maven Project");
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        Container contentPane = frame.getContentPane();

        contentPane.setLayout(new BoxLayout(contentPane, BoxLayout.Y_AXIS));

        JPanel inputPanel = new JPanel();
        inputPanel.setLayout(new BoxLayout(inputPanel, BoxLayout.Y_AXIS));
        contentPane.add(inputPanel);

        JPanel namePanel = new JPanel();
        inputPanel.add(namePanel);
        namePanel.add(new JLabel("Name:"));
        final JTextField name = new JTextField();
        name.setColumns(30);
        namePanel.add(name);

        JPanel versionPanel = new JPanel();
        inputPanel.add(versionPanel);
        versionPanel.add(new JLabel("Version:"));
        final JTextField version = new JTextField();
        version.setColumns(30);
        versionPanel.add(version);

        JPanel sDescriptonPanel = new JPanel();
        inputPanel.add(sDescriptonPanel);
        sDescriptonPanel.add(new JLabel("Short Description:"));
        final JTextField shortDescription = new JTextField();
        shortDescription.setColumns(30);
        sDescriptonPanel.add(shortDescription);

        JPanel lDescriptionPanel = new JPanel();
        inputPanel.add(lDescriptionPanel);
        lDescriptionPanel.add(new JLabel("Long Description:"));
        final JTextField longDescription = new JTextField();
        longDescription.setColumns(30);
        lDescriptionPanel.add(longDescription);

        JPanel artifactPanel = new JPanel();
        inputPanel.add(artifactPanel);
        artifactPanel.add(new JLabel("Artifact ID:"));
        final JTextField artifactId = new JTextField();
        artifactId.setColumns(30);
        artifactPanel.add(artifactId);
        artifactId.setText(stripExtension(moduleFile));

        JPanel extendingPanel = new JPanel();
        inputPanel.add(extendingPanel);
        extendingPanel.add(new JLabel("Extends"));
        final JTextField extendingProject = new JTextField();
        extendingProject.setColumns(30);
        extendingProject.setText("../test/project.xml");
        extendingPanel.add(extendingProject);

        JPanel repositoryPanel = new JPanel();
        inputPanel.add(repositoryPanel);
        repositoryPanel.add(new JLabel("Repository"));
        final JTextField repository = new JTextField();
        repository.setColumns(30);
        repository.setText("scm:cvs:pserver:${cvs.username}@slc-codebuild:/[CVS_ROOT]:[PATH]");
        repositoryPanel.add(repository);

        // Button Panel
        JPanel buttonPanel = new JPanel();
        contentPane.add(buttonPanel);

        // OK Button
        JButton okButton = new JButton("OK");
        buttonPanel.add(okButton);
        okButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                CreateProjectXML.this.name = name.getText();
                CreateProjectXML.this.version = version.getText();
                CreateProjectXML.this.shortDescription = shortDescription.getText();
                CreateProjectXML.this.longDescription = longDescription.getText();
                CreateProjectXML.this.artifactId = artifactId.getText();
                CreateProjectXML.this.extendingProject = extendingProject.getText();
                CreateProjectXML.this.repository = repository.getText();

                frame.dispose();

                try
                {
                    IntelliJModule module = new IntelliJModule(moduleFile);
                    isWebModule = module.isWebModule();
                    dependencies = module.getDependencies();
                    writeDependencies();
                }
                catch (Exception ex)
                {
                    ex.printStackTrace();
                }
            }
        });

        // Cancel Button
        JButton cancelButton = new JButton("Cancel");
        buttonPanel.add(cancelButton);
        cancelButton.addActionListener(new ActionListener()
        {
            public void actionPerformed(ActionEvent e)
            {
                System.exit(0);
            }
        });

        //Display the window.
        frame.pack();
        frame.setLocationRelativeTo(null);
        frame.setVisible(true);
    }

    /**
     * Removes the extension from the file.
     *
     * @param file file to remove extension from
     * @return name of the file without the extension
     */
    private String stripExtension(File file)
    {
        String fileName = file.getName();
        int index = fileName.lastIndexOf(".");
        if(index > -1)
        {
            fileName = fileName.substring(0, index);
        }
        return fileName;
    }

    public static void main(String[] args)
    {
        File moduleFile = new File(args[0]);
        if(!moduleFile.exists())
        {
            System.out.println("Module moduleFile does not exist: " + moduleFile.getAbsolutePath());
            System.exit(1);
        }

        CreateProjectXML createProject = new CreateProjectXML(moduleFile);
    }
}
