package intellijmaven.tools;

import intellijmaven.core.Dependency;
import intellijmaven.core.IMavenProject;
import intellijmaven.core.IntelliJModule;
import intellijmaven.core.MavenProject;
import org.jdom.JDOMException;

import java.io.File;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

/**
 * Updates the IntelliJ module file based on the Maven project file. Removes any dependencies
 * and reads them. Thus each time you update dependencies in Maven, the IntelliJ module (.iml)
 * can be updated.
 *
 * @author jsabin
 */
public class UpdateIntelliJModule
{
    public static void update(IntelliJModule module, IMavenProject maven) throws IOException, JDOMException
    {
        module.removeAllDependencies();

        List dependencies = maven.getDependencies();
        for(Iterator iterator = dependencies.iterator(); iterator.hasNext();)
        {
            Dependency dependency = (Dependency)iterator.next();
            module.addDependency(dependency);
        }

        module.save();
    }

    public static void update(File moduleFile, File projectFile) throws IOException, JDOMException
    {
        IntelliJModule module = new IntelliJModule(moduleFile);
        IMavenProject maven = new MavenProject(projectFile, module.isWebModule());

        update(module, maven);
    }

    private static void help()
    {
        System.out.println("\nUpdateIntelliJModule syncs your IntelliJ 5.0 module (.iml) file with your Maven" +
                " project file. It removes existing dependencies and re-adds them based on your Maven project.xml" +
                " file. It also ensures that the depedencies are setup to be copied to the exploded directory.\n");
        System.out.println("Usage: java -classpath classes;lib/jdom.jar intellijmaven.tools.UpdateIntelliJModule <projectXMLPath> <ideadModulePath>\n");
        System.out.println("\twhere");
        System.out.println("\t<projectXMLPath> is the path to the Maven project.xml file.\n");
        System.out.println("\t<ideadModulePath> is the path to the IntelliJ module file (.iml).\n");

    }

    public static void main(String[] args) throws Exception
    {
        if (args.length < 2)
        {
            help();
        }
        else
        {
            File projectFile = new File(args[0]);
            if (!projectFile.exists())
            {
                System.err.println("Maven project xml file does not exist: " + projectFile.getAbsolutePath() + "\n\n");
                help();
                System.exit(1);
            }

            File moduleFile = new File(args[1]);
            if (!projectFile.exists())
            {
                System.err.println("Module file does not exist: " + moduleFile.getAbsolutePath() + "\n\n");
                help();
                System.exit(1);
            }

            UpdateIntelliJModule.update(moduleFile, projectFile);
        }
    }
}
